/**
 * @file m13_rza2m_ek_lcd_settings.c
 *
 * @brief LCD and TCON signals settings
 *
 *
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Dependencies
#include "m13_rza2m_ek_lcd_settings.h"

// Private global variables and functions

// TCON timing setting, VS
static const vdc_lcd_tcon_timing_t lcd_tcon_timing_vs =
{
    (uint16_t) (0 * 2u),                     // Signal pulse start position
    (uint16_t) LCD_DISP_W_VSYNC,             // Pulse width
    VDC_LCD_TCON_POLMD_NORMAL,               // Signal operating reference select
    VDC_LCD_TCON_REFSEL_HSYNC,               // Signal generation mode select
    VDC_SIG_POL_INVERTED,                    // Polarity inversion control of signal
    VDC_LCD_TCON_PIN_2,                      // Output pin for LCD driving signal
    VDC_EDGE_RISING                          // Output phase control of signal
};

// TCON timing setting, HS
static const vdc_lcd_tcon_timing_t lcd_tcon_timing_hs =
{
    (uint16_t) 0u,                           // Signal pulse start position
    (uint16_t) LCD_DISP_W_HSYNC,             // Pulse width
    VDC_LCD_TCON_POLMD_NORMAL,               // Signal operating reference select
    VDC_LCD_TCON_REFSEL_HSYNC,               // Signal generation mode select
    VDC_SIG_POL_INVERTED,                    // Polarity inversion control of signal
    VDC_LCD_TCON_PIN_1,                      // Output pin for LCD driving signal
    VDC_EDGE_RISING                          // Output phase control of signal
};

// TCON timing setting, DE - none
static const vdc_lcd_tcon_timing_t lcd_tcon_timing_de_ =
{
    (uint16_t) 0,                            // Signal pulse start position
    (uint16_t) 0,                            // Pulse width
    VDC_LCD_TCON_POLMD_NORMAL,               // Signal operating reference select
    VDC_LCD_TCON_REFSEL_HSYNC,               // Signal generation mode select
    VDC_SIG_POL_NOT_INVERTED,                // Polarity inversion control of signal
    VDC_LCD_TCON_PIN_NON,                    // Output pin for LCD driving signal
    VDC_EDGE_RISING                          // Output phase control of signal
};

// TCON timing setting, VE - none
static const vdc_lcd_tcon_timing_t lcd_tcon_timing_ve =
{
    (uint16_t) (LCD_CH0_DISP_VS * 2),        // Signal pulse start position
    (uint16_t) (LCD_CH0_DISP_VW * 2),        // Pulse width
    VDC_LCD_TCON_POLMD_NORMAL,               // Signal operating reference select
    (uint16_t) 0,                            // Signal generation mode select
    (uint16_t) 0,                            // Polarity inversion control of signal
    VDC_LCD_TCON_PIN_NON,                    // Output pin for LCD driving signal
    VDC_EDGE_RISING                          // Output phase control of signal
};

// TCON timing setting, HE - none
static const vdc_lcd_tcon_timing_t lcd_tcon_timing_he =
{
    (uint16_t) LCD_CH0_DISP_HS,              // Signal pulse start position
    (uint16_t) LCD_CH0_DISP_HW,              // Pulse width
    VDC_LCD_TCON_POLMD_NORMAL,               // Signal operating reference select
    (uint16_t) 0,                            // Signal generation mode select
    (uint16_t) 0,                            // Polarity inversion control of signal
    VDC_LCD_TCON_PIN_NON,                    // Output pin for LCD driving signal
    VDC_EDGE_RISING                          // Output phase control of signal
};

/**
 * @name       bsp_lcd_tcon_Setting_ch0
 * @brief      Configure M13-RZA2M-EK LCD TCON signals
 * @param[in]  vdc_lcd_tcon_timing_t  outctrl - Timing control parameters to set
 * @return     None
 */
void bsp_lcd_tcon_setting_ch0(const vdc_lcd_tcon_timing_t **outctrl)
{
    if(0 == outctrl)
    {
        return;
    }
    *outctrl++ = &lcd_tcon_timing_vs;        // STVA/VS:
    *outctrl++ = &lcd_tcon_timing_ve;        // STVB/VE:
    *outctrl++ = &lcd_tcon_timing_hs;        // STH/SP/HS:
    *outctrl++ = &lcd_tcon_timing_he;        // STB/LP/HE:
    *outctrl++ = NULL;                       // CPV/GCK: Not used
    *outctrl++ = NULL;                       // POLA: Not used
    *outctrl++ = NULL;                       // POLB: Not used
    *outctrl = &lcd_tcon_timing_de_;         // DE: Not used
}

/**
 * @name       bsp_set_lcd_tcon_settings
 * @brief      Set timing control parameters in VDC5 for the LCD panel.
 * @param[in]  outctrl - Timing control parameters to set
 * @return     None
 */
void bsp_set_lcd_tcon_settings(vdc_channel_t channel, const vdc_lcd_tcon_timing_t **outctrl)
{
    /* LCD TCON timing setting */
    if(channel == VDC_CHANNEL_0)
    {
        bsp_lcd_tcon_setting_ch0(outctrl);
    }
}


