/**
 * @file m13_rza2m_ek_acc.c
 * @brief M13-RZ/A2M-EK 3-axis acceleration sensor definiton (Wurth Elektronik WSEN-ITDS )
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http:// www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "m13_rza2m_ek_config.h"
#define TRACE_LEVEL BSP_CONFIG_TRACE_LEVEL_ACC

// Dependencies
#include "m13_rza2m_ek_acc.h"
#include "m13_rza2m_ek_vcom.h"


// Macro definitions
#define WSEN_ITDS_SLAVE_ADDR_W 0x32
#define WSEN_ITDS_SLAVE_ADDR_R 0x33

// Global variables
static int_t acc_handle;

/**
 * @brief WSEN-ITDS initialization
 * @return Error code (see #ErrorCode enumeration)
 **/
bsp_status_t bsp_acc_init(void)
{
    bsp_status_t error;
    uint8_t value;

    // Open I2C channel 3 and initialize accelerometer handler
    acc_handle = bsp_i2c_open(BSP_I2C_CHANNEL_3);

	if (acc_handle == BSP_I2C_INVALID_HANDLE)
	{
		return BSP_STATUS_INVALID_CHANNEL;
	}

    // Read DEVICE_ID register
    error = bsp_acc_read_reg(WSEN_ITDS_DEVICE_ID, &value);

    if(error)
    {
        return error;
    }

    // Verify device identifier
    if(value != WSEN_ITDS_DEVICE_ID_DEFAULT)
    {
        PRINTF_INFO("Mem ID read = 0x%02X instead of 0x44 \r\n", value);
        return BSP_STATUS_INVALID_VERSION;
    }


    // Set normal operation mode
    value = 0;
    value |= WSEN_ITDS_CTRL_1_ODR_400_HZ;
    value |= WSEN_ITDS_CTRL_1_MODE_NORMAL;
    value |= WSEN_ITDS_CTRL_1_LP_MODE_NORMAL;


    error = bsp_acc_write_reg(WSEN_ITDS_CTRL_1, value);

    if(error)
    {
        PRINTF_INFO("Could not set Operation Mode \r\n");
        return error;
    }

    // Enable block data update and address auto increment
    value = 0;
    value |= WSEN_ITDS_CTRL_2_BDU;
    value |= WSEN_ITDS_CTRL_2_IF_ADD_INC;

    error = bsp_acc_write_reg(WSEN_ITDS_CTRL_2, value);

    if(error)
    {
        PRINTF_INFO("Could not Enable block data \r\n");
        return error;
    }

    // Select bandwidth and full scale
    value = 0;
    value |= WSEN_ITDS_CTRL_6_BW_FILT_ODR_DIV_2;
    value |= WSEN_ITDS_CTRL_6_FS_2G;

    error = bsp_acc_write_reg(WSEN_ITDS_CTRL_6, value);

    if(error)
    {
        PRINTF_INFO("Could not Select bandwidth and full scale \r\n");
        return error;
    }

    // Successful initialization
    PRINTF_INFO(" Accelerometer Init Success!\r\n");
    return BSP_STATUS_OK;
}

/**
 * @brief Get acceleration data
 * @param[out] ax Acceleration value (X axis)
 * @param[out] ay Acceleration value (Y axis)
 * @param[out] az Acceleration value (Z axis)
 * @param[in] data Register value
 * @return Error code (see #ErrorCode enumeration)
 **/
bsp_status_t bsp_acc_get_acc(int16_t *ax, int16_t *ay, int16_t *az)
{
    bsp_status_t error;
    uint8_t read_value[6] = {0, 0, 0, 0, 0, 0};
    uint16_t temp;

    error = bsp_acc_read_regs(WSEN_ITDS_X_OUT_L, &read_value[0], sizeof(read_value));

    // Form x,y,z axis read data to 16bits
    temp = ((read_value[1] << 8) + read_value[0]);
    *ax = (int16_t)temp;

    temp = ((read_value[3] << 8) + read_value[2]);
    *ay = (int16_t)temp;

    temp = ((read_value[5] << 8) + read_value[4]);
    *az = (int16_t)temp;

    //PRINTF_DEBUG(" ax = %d \r\n", *ax);
    PRINTF_DEBUG(" ay = %d \r\n", *ay);
    //PRINTF_DEBUG(" az = %d \r\n", *az);
    // Return status code
    return (error);
}

/**
 * @brief Write WSEN-ITDS register
 * @param[in] address Register address
 * @param[in] data Register value
 * @return Error code (see #ErrorCode enumeration)
 **/
bsp_status_t bsp_acc_write_reg(uint8_t address, uint8_t data)
{
    bsp_status_t error;

    error = bsp_i2c_write (acc_handle, BSP_I2C_SLAVE_ADDR_ACC, address, 1, &data, 1 );

    // Return status code
    return (error);
}


/**
 * @brief Read WSEN-ITDS register
 * @param[in] address Register address
 * @param[out] data Register value
 * @return Error code (see #ErrorCode enumeration)
 **/
bsp_status_t bsp_acc_read_reg(uint8_t address, uint8_t *data)
{
    bsp_status_t error;

    error = bsp_i2c_read (acc_handle, BSP_I2C_SLAVE_ADDR_ACC, address, 1, data, 1 );

    // Return status code
    return (error);
}

bsp_status_t bsp_acc_read_regs(uint8_t address, uint8_t *data, size_t data_length)
{
    bsp_status_t error;

    error = bsp_i2c_read (acc_handle, BSP_I2C_SLAVE_ADDR_ACC, address, 1, data, data_length);

    // Return status code
    return (error);
}
