/**
 * @file m13_rza2m_ek_i2c.c
 * @brief M13-RZ/A2M-EK I2C BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "r_gpio_drv_api.h"
#include "r_riic_drv_api.h"
#include "m13_rza2m_ek_i2c.h"


// Macro definitions
#define RIIC_CH0_RZ_A2M (DEVICE_INDENTIFIER "riic0")
#define RIIC_CH1_RZ_A2M (DEVICE_INDENTIFIER "riic1")
#define RIIC_CH2_RZ_A2M (DEVICE_INDENTIFIER "riic2")
#define RIIC_CH3_RZ_A2M (DEVICE_INDENTIFIER "riic3")

// Global variables and functions

// Private global variables and functions

/**
 * @name       bsp_i2c_init
 * @brief      Configure M13-RZA2M-EK RIIC pins channels
 * @param[in]  uint8_t riic_channel : riic channel number
 * @return     None
 */
bsp_status_t bsp_i2c_init(bsp_i2c_channel_t channel)
{
    int_t gpio_handle, err;
		
    gpio_handle = direct_open("gpio", 0);

    if(gpio_handle < 0)
    {
		return BSP_STATUS_FAILURE;
    }

    if(channel == BSP_I2C_CHANNEL_0)
    {
        // Configure RIIC0 pin
        err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_riic0_pin_table);

        if(err < 0)
        {
		    return BSP_STATUS_PIN_CONFIG_FAILURE;
        }
    }
    else if(channel == BSP_I2C_CHANNEL_1)
    {
        // Configure RIIC1 pin
        err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_riic1_pin_table);

        if(err < 0)
        {
            return BSP_STATUS_PIN_CONFIG_FAILURE;
        }
    }
    else if(channel == BSP_I2C_CHANNEL_2)
    {
        // Configure RIIC2 pin
        err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_riic2_pin_table);

        if(err < 0)
        {
            return BSP_STATUS_PIN_CONFIG_FAILURE;
        }
    }
    else if(channel == BSP_I2C_CHANNEL_3)
    {
        // Configure RIIC3 pin
        err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_riic3_pin_table);

        if(err < 0)
        {
            return BSP_STATUS_PIN_CONFIG_FAILURE;
        }
    }
    else
    {
    	return BSP_STATUS_PIN_CONFIG_FAILURE;
    }

    return BSP_STATUS_OK;
}

bsp_i2c_handle_t bsp_i2c_open(bsp_i2c_channel_t channel)
{
	bsp_i2c_handle_t i2c_handle = BSP_I2C_INVALID_HANDLE;

    if(channel == BSP_I2C_CHANNEL_0)
    {
    	i2c_handle = open(RIIC_CH0_RZ_A2M, O_RDWR);
    }
    else if(channel == BSP_I2C_CHANNEL_1)
    {
    	i2c_handle = open(RIIC_CH1_RZ_A2M, O_RDWR);
    }
    else if(channel == BSP_I2C_CHANNEL_2)
    {
    	i2c_handle = open(RIIC_CH2_RZ_A2M, O_RDWR);
    }
    else if(channel == BSP_I2C_CHANNEL_3)
    {
    	i2c_handle = open(RIIC_CH3_RZ_A2M, O_RDWR);
    }
    else
    {
    	return BSP_STATUS_PIN_CONFIG_FAILURE;
    }

    return i2c_handle;
}

/**
 * @name       bsp_i2c_write
 * @brief      Perform an i2c write access
 * @param[in]  Handle for RZ/A2M device control function
 * @param[in]  Device_address
 * @param[in]  Sub_address
 * @param[in]  Sub_address_length
 * @param[in]  Data
 * @param[in]  Data_length
 * @return     error code
 */
bsp_status_t bsp_i2c_write(bsp_i2c_handle_t handle, uint8_t device_address, uint32_t sub_address,
    uint32_t sub_address_length, const uint8_t *data, size_t data_length)
{
    uint32_t delay = 200000;

    st_r_drv_riic_transfer_t i2c_write;

    i2c_write.device_address = (device_address << 1);

    if(sub_address_length == 1)
    {
        i2c_write.sub_address_type = RIIC_SUB_ADDR_WIDTH_8_BITS;
    }
    else if(sub_address_length == 2)
    {
        i2c_write.sub_address_type = RIIC_SUB_ADDR_WIDTH_16_BITS;
    }
    else if(sub_address_length == 3)
    {
        i2c_write.sub_address_type = RIIC_SUB_ADDR_WIDTH_32_BITS;
    }
    else
    {
        i2c_write.sub_address_type = RIIC_SUB_ADDR_NONE;
    }

    i2c_write.sub_address = sub_address;
    i2c_write.number_of_bytes = data_length;
    i2c_write.p_data_buffer = (uint8_t *) data;

    if(DRV_ERROR >= control(handle, CTL_RIIC_WRITE, &i2c_write))
    {
        return BSP_STATUS_FAILURE;
    }

    while(delay--)
    {
        __asm__ ("nop");
    }
    /* Return status code */
    return BSP_STATUS_OK;
}

/**
 * @name       bsp_i2c_read
 * @brief      Perform an i2c read access
 * @param[in]  Handle for RZ/A2M device control function
 * @param[in]  Device_address
 * @param[in]  Sub_address
 * @param[in]  Sub_address_length
 * @param[in]  Data
 * @param[in]  Data_length
 * @return     error code
 */
bsp_status_t bsp_i2c_read(bsp_i2c_handle_t handle, uint8_t device_address, uint32_t sub_address,
    uint32_t sub_address_length, uint8_t *data, size_t data_length)
{
    bsp_status_t error = BSP_STATUS_OK;

    st_r_drv_riic_transfer_t i2c_read;
    i2c_read.device_address = (device_address << 1);

    if(sub_address_length == 1)
    {
        i2c_read.sub_address_type = RIIC_SUB_ADDR_WIDTH_8_BITS;
    }
    else if(sub_address_length == 2)
    {
        i2c_read.sub_address_type = RIIC_SUB_ADDR_WIDTH_16_BITS;
    }
    else if(sub_address_length == 3)
    {
        i2c_read.sub_address_type = RIIC_SUB_ADDR_WIDTH_32_BITS;
    }
    else
    {
        i2c_read.sub_address_type = RIIC_SUB_ADDR_NONE;
    }

    i2c_read.sub_address = sub_address;
    i2c_read.number_of_bytes = data_length;
    i2c_read.p_data_buffer = data;

    if(DRV_ERROR >= control(handle, CTL_RIIC_READ, &i2c_read))
    {
        error = BSP_STATUS_FAILURE;
    }

    return (error);
}
