/**
 * @file m13_rza2m_ek_lcd.c
 * @brief M13-RZ/A2M-EK lcd BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Dependencies
#include "r_intc_lld_rza2m.h"
#include "m13_rza2m_ek_lcd.h"
#include "m13_rza2m_ek_lcd_graphics.h"
#include "m13_rza2m_ek_lcd_settings.h"

// Global variables and functions
uint16_t background_color;
uint16_t screen_rotation;
int_t gpio_handle;

// Private global variables and functions
static uint16_t text_color;                                 // Current text color
static font_t *current_font;                                // Current Font
static vdc_int_t interrupt;                                 // Interrupt parameter
static vdc_read_chg_t read_chg;                             // Read data parameter
static vdc_gr_disp_sel_t gr_disp_sel[VDC_GR_TYPE_NUM];      // Graphics display mode
static vdc_period_rect_t period_rect;                       // Display area
static volatile uint32_t vsync_count;

static const st_r_drv_gpio_pin_rw_t back_light_on  = { GPIO_PORT_J_PIN_7, GPIO_LEVEL_HIGH };
static const st_r_drv_gpio_pin_rw_t back_light_off    = { GPIO_PORT_J_PIN_7, GPIO_LEVEL_LOW  };
static const st_r_drv_gpio_pin_rw_t display_on     = { GPIO_PORT_7_PIN_7, GPIO_LEVEL_HIGH };
static const st_r_drv_gpio_pin_rw_t display_off    = { GPIO_PORT_7_PIN_7, GPIO_LEVEL_LOW  };

// LCD Frame buffers (these buffers are stored in the uncached Ram)
static uint8_t frame_buffer_1[LCD_HEIGHT * LCD_STRIDE] __attribute__ ((section(BSP_LCD_RAM_SECTION)));
static uint8_t frame_buffer_2[LCD_HEIGHT * LCD_STRIDE] __attribute__ ((section(BSP_LCD_RAM_SECTION)));

static vdc_frame_buffer_t frame_buffer =
{ frame_buffer_1, frame_buffer_2, frame_buffer_1};

static uint16_t *p = (uint16_t *)&frame_buffer_1[0];

/**
 * @name       vsync_callback
 * @brief      Callback for the VDC vsync interrupt
 * @param[in]  int_type - Unused
 * @return     None
 */
static void vsync_callback(vdc_int_type_t int_type)
{
    if(vsync_count > 0)
    {
        vsync_count--;
    }
}

/**
 * @name       wait_vsync
 * @brief      Wait for a number of v-sync interrupt periods
 * @param[in]  wait_count - number of v-sync interrupts to wait
 * @return     None
 */
static void wait_vsync(uint32_t wait_count)
{
    vsync_count = wait_count;

    while(vsync_count > 0)
    {
        // Do nothing
    }
}


/**
 * @name       bsp_lcd_io_init
 * @brief      Configure the lcd pins
 * @param[in]  None
 * @return     None
 */
bsp_status_t bsp_lcd_io_init(void)
{
    int_t err;
	
    gpio_handle = direct_open("gpio", 0);
    if(gpio_handle < 0)
    {
		return BSP_STATUS_FAILURE;
    }

    err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_lcd_pin_table);

    if(err < 0)
    {
        return BSP_STATUS_PIN_CONFIG_FAILURE;
    }
	
    return BSP_STATUS_OK;	
}

/**
 * @name       bsp_lcd_init
 * @brief      Set default LCD settings and configures system to start displaying on the LCD panel
 * @param[in]  None
 * @return     None
 */
void bsp_lcd_init(void)
{
    // Configure the board's lcd pins
    bsp_lcd_io_init();

    // Set screen rotation to default rotation ROTATION_ZERO
    screen_rotation = ROTATION_ZERO;

    // Configures VDC system and start displaying on the LCD panel
    uint32_t index;
    vdc_error_t error;
    vdc_channel_t ch               = VDC_CHANNEL_0;
    vdc_layer_id_t layer_id          = VDC_LAYER_ID_2_RD;
    vdc_graphics_type_t graphics_type    = VDC_GR_TYPE_GR2;
    vdc_gr_format_t gr_format        = VDC_GR_FORMAT_RGB565;

    read_chg.gr_base        = NULL;         // Frame buffer base address
    read_chg.width_read_fb  = NULL;         // Width of the image read from frame buffer
    read_chg.gr_grc         = NULL;         // Display area
    read_chg.gr_disp_sel    = NULL;         // Graphics display mode

    for(index = 0; index < (uint32_t)VDC_GR_TYPE_NUM; index++)
    {
        gr_disp_sel[index] = VDC_DISPSEL_IGNORED;

    }

    period_rect.vs  = (uint16_t)LCD_CH0_DISP_VS;
    period_rect.vw  = (uint16_t)LCD_CH0_DISP_VW;
    period_rect.hs  = (uint16_t)LCD_CH0_DISP_HS;
    period_rect.hw  = (uint16_t)LCD_CH0_DISP_HW;

    // Configure interrupt
    bsp_lcd_config_interrupt(ch);

    // Graphics initialization processing
    error = bsp_graphic_init(NULL);

    if(VDC_OK != error)
    {
        return;
    }

    error = bsp_graphic_create_surface(
        layer_id,
        frame_buffer.pcurrent,
        LCD_STRIDE,
        gr_format,
        &period_rect
        );

    if(VDC_OK != error)
    {
        return;
    }

    interrupt.type        = VDC_INT_TYPE_VLINE;
    interrupt.callback     = vsync_callback;
    interrupt.line_num    = 0u;

    // Set VDC5 interrupt
    error = R_VDC_CallbackISR(ch, &interrupt);

    if(VDC_OK != error)
    {
        return;
    }

    gr_disp_sel[graphics_type] = VDC_DISPSEL_CURRENT;
    error = bsp_graphic_start_surfaces(gr_disp_sel);

    if(VDC_OK != error)
    {
        return;
    }

    // Turn on the display
    bsp_lcd_display_on();

    // Turn on the back light
    bsp_lcd_backlight_on();
}

/**
 * @name       bsp_lcd_deinit
 * @brief      Terminate graphic process
 * @param[in]  None
 * @return     None
 */
void bsp_lcd_deinit(void)
{
    bsp_graphic_term();
}

/**
 * @name       bsp_lcd_config_interrupt
 * @brief      Configure LCD interrupt
 * @param[in]  None
 * @return     None
 */
void bsp_lcd_config_interrupt(vdc_channel_t ch)
{
    void (*func)(uint32_t);
    uint32_t ret_code;

    func = R_VDC_GetISR(ch, VDC_INT_TYPE_VLINE);
    ret_code = R_INTC_RegistIntFunc(INTC_ID_VDC6_GR3_VLINE0, func);
    if(0 != ret_code)
    {
        return;
    }

    ret_code = R_INTC_SetPriority(INTC_ID_VDC6_GR3_VLINE0, 5u);
    if(0 != ret_code)
    {
        return;
    }

    ret_code = R_INTC_Enable(INTC_ID_VDC6_GR3_VLINE0);

    if(0 != ret_code)
    {
        return;
    }
}

/**
 * @name       bsp_lcd_display_on
 * @brief      Assert LCD DISP's pin to turn on display on the LCD side
 * @param[in]  int_type - Unused
 * @return     None
 */
void bsp_lcd_display_on(void)
{
    direct_control(gpio_handle, CTL_GPIO_PIN_WRITE, (void *) &display_on);
}

/**
 * @name       bsp_lcd_display_on
 * @brief      De-assert LCD DISP's pin to turn off display on the LCD side
 * @param[in]  int_type - Unused
 * @return     None
 */
void bsp_lcd_display_off(void)
{
    direct_control(gpio_handle, CTL_GPIO_PIN_WRITE, (void *) &display_off);
}

/**
 * @name       bsp_lcd_clear
 * @brief      Fill in the LCD with parameter color
 * @param[in]  bsp_color_t color
 * @return     None
 */
void bsp_lcd_clear(bsp_color_t color)
{
    for(uint32_t i = 0; i < (LCD_WIDTH * LCD_HEIGHT); i++)
    {
        p[i] = color;
    }
}

/**
 * @name       bsp_lcd_backlight_on
 * @brief      Turn on backlight
 * @param[in]  None color
 * @return     None
 */
void bsp_lcd_backlight_on(void)
{
    direct_control(gpio_handle, CTL_GPIO_PIN_WRITE, (void *) &back_light_on);
}

/**
 * @name       bsp_lcd_backlight_off
 * @brief      Turn off backlight
 * @param[in]  None color
 * @return     None
 */
void bsp_lcd_backlight_off(void)
{
    direct_control(gpio_handle, CTL_GPIO_PIN_WRITE, (void *) &back_light_off);
}

/**
 * @name       bsp_lcd_read_pixel
 * @brief      Read pixel color value at given x/y coordonates
 * @param[in]  uint16_t x, y coordonates
 * @return     Return pixel color
 */
bsp_color_t bsp_lcd_read_pixel(uint16_t x, uint16_t y)
{
    bsp_color_t color;
    return color;
}

/**
 * @name       bsp_lcd_draw_pixel
 * @brief      Draw pixel at given x/y coordonates
 * @param[in]  uint16_t x, y coordonates, pixel color
 * @return     None
 */
void bsp_lcd_draw_pixel(uint16_t x, uint16_t y, bsp_color_t color)
{
    // Verify pixel is within LCD range
    if( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        switch(screen_rotation)
        {
        case ROTATION_ZERO:
        {
            p[(y * LCD_WIDTH) + x] = color;
            break;
        }
        case ROTATION_180:
        {
            p[((LCD_HEIGHT - y) * LCD_WIDTH) + (LCD_WIDTH - x)] = color;
            break;
        }
        default:
        {
            p[(y * LCD_WIDTH) + x] = color;
            break;
        }
        }
    }
}

/**
 * @name       bsp_lcd_draw_horizontal_line
 * @brief      Draw horizontal line
 * @param[in]  uint16_t x, y coordonates, uint16_t length, bsp_color_t pixel color
 * @return     None
 */
void bsp_lcd_draw_horizontal_line(uint16_t x, uint16_t y, uint16_t length, bsp_color_t color)
{
    // Verify starting point of horizontal line is within LCD range
    if( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending point is over LCD range, reduce line length to fit LCD range
        if((x + length) > LCD_WIDTH)
        {
            length = LCD_WIDTH - x;
        }
        // Draw line
        for(uint16_t i = 0; i <= length; i++)
        {
            bsp_lcd_draw_pixel(x, y, color);
            x++;
        }
    }
}

/**
 * @name       bsp_lcd_draw_horizontal_line
 * @brief      Draw vertical line
 * @param[in]  uint16_t x, y coordonates, uint16_t length, bsp_color_t pixel color
 * @return     None
 */
void bsp_lcd_draw_vertical_line(uint16_t x, uint16_t y, uint16_t length, bsp_color_t color)
{
    // Verify starting point of vertical line is within LCD range
    if( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending point is over LCD range, reduce line length to fit LCD range
        if((y + length) > LCD_HEIGHT)
        {
            length = LCD_HEIGHT - y;
        }
        // Draw line
        for(uint16_t i = 0; i < length; i++)
        {
            bsp_lcd_draw_pixel(x, y, color);
            y++;
        }
    }
}

/**
 * @name       bsp_lcd_draw_rect
 * @brief      Draw rectangle
 * @param[in]  uint16_t x, y coordonates, uint16_t width, height, bsp_color_t pixel color
 * @return     None
 */
void bsp_lcd_draw_rect(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bsp_color_t color)
{
    // Verify starting point of vertical line is within LCD range
    if( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending width point is over LCD range, reduce width to fit LCD range
        if((x + width) >= LCD_WIDTH)
        {
            width = LCD_WIDTH - x;
        }

        // If ending height point is over LCD range, reduce height to fit LCD range
        if((y + height) >= LCD_HEIGHT)
        {
            height = LCD_HEIGHT - y;
        }

        // Draw horizontal line 1
        bsp_lcd_draw_horizontal_line(x, y, width, color);

        // Draw horizontal line 2
        bsp_lcd_draw_horizontal_line(x, (y + height), width, color);

        // Draw vertical line 1
        bsp_lcd_draw_vertical_line(x, y, height, color);

        // Draw vertical line 2
        bsp_lcd_draw_vertical_line((x + width), y, height, color);
    }
}

/**
 * @name       bsp_lcd_fill_rect
 * @brief      Draw and fill rectangle
 * @param[in]  uint16_t x, y coordonates, uint16_t width, height, bsp_color_t pixel color
 * @return     None
 */
void bsp_lcd_fill_rect(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bsp_color_t color)
{
    // Verify starting point of vertical line is within LCD range
    if( (x < LCD_WIDTH) && (y < LCD_HEIGHT) )
    {
        // If ending width point is over LCD range, reduce width to fit LCD range
        if((x + width) >= LCD_WIDTH)
        {
            width = LCD_WIDTH - x;
        }

        // If ending height point is over LCD range, reduce height to fit LCD range
        if((y + height) >= LCD_HEIGHT)
        {
            height = LCD_HEIGHT - y;
        }
        // Fill rectangle with height number of lines
        for(uint16_t i = 0; i < height; i++)
        {
            bsp_lcd_draw_horizontal_line(x, y, width, color);
            y++;
        }
    }
}

/**
 * @name       bsp_lcd_set_text_background_color
 * @brief      Set text background color for non-text pixel
 * @param[in]  bsp_color_t pixel color
 * @return     None
 */
void bsp_lcd_set_text_background_color(bsp_color_t color)
{
    background_color = color;
}

/**
 * @name       bsp_lcd_set_text_color
 * @brief      Set text color for text pixel
 * @param[in]  bsp_color_t pixel color
 * @return     None
 */
void bsp_lcd_set_text_color(bsp_color_t color)
{
    text_color = color;
}

/**
 * @name       bsp_lcd_set_font
 * @brief      Select default font
 * @param[in]  font_t *p_font
 * @return     None
 */
void bsp_lcd_set_font(font_t *p_font)
{
    current_font = p_font;
}

/**
 * @name       bsp_lcd_display_char
 * @brief      Display character font at given coordonates
 * @param[in]  uint16_t x,y coordonates, font_t *p_font
 * @return     None
 */
void bsp_lcd_display_char(uint16_t x, uint16_t y, uint8_t c)
{
    uint16_t index, color, char_width;
    uint16_t pixel_state;

    // Verify character is between 0x20 et 0x7E
    if(c >= 32 && c <= 126)
    {
        char_width = (current_font->width + 7) / 8;

        // Position the font index to the desired character
        index = (c - 32) * char_width * current_font->height;

        // Go through each of the 8 lines of the character
        for(uint8_t j = 0; j < current_font->height; j++)
        {
            // Draw each of the X pixels of the character line
            for(uint8_t i = 0; i <= (current_font->width); i++)
            {
                // Check which pixel is to be drawn or not and choose color for each of them
                pixel_state = current_font->table [ index + (char_width * j) + (i / 8) ] & (0x80 >> (i % 8));

                if(pixel_state)
                {
                    color = text_color;
                }
                else
                {
                    color = background_color;
                }

                // Draw each pixel
                bsp_lcd_draw_pixel(x + i, y + j, color);
            }
        }
    }
}

/**
 * @name       bsp_lcd_display_string
 * @brief      Display character string at given coordonates
 * @param[in]  uint16_t x,y coordonates, const char * s
 * @return     None
 */
void bsp_lcd_display_string(uint16_t x, uint16_t y, const char *s)
{
    while(*s != '\0')
    {
        bsp_lcd_display_char(x, y, *s);
        x+=current_font->width;
        s++;
    }
}


void bsp_lcd_enable_double_buffer(void)
{

    bsp_lcd_set_display_buffer(1);
    bsp_lcd_set_working_buffer(2);
}

void bsp_lcd_swap_buffer(void)
{
    if(frame_buffer.pcurrent == frame_buffer.pb1)
    {
        bsp_lcd_set_display_buffer(2);
        bsp_lcd_set_working_buffer(1);

        frame_buffer.pcurrent = frame_buffer.pb2;
    }
    else
    {
        bsp_lcd_set_display_buffer(1);
        bsp_lcd_set_working_buffer(2);

        frame_buffer.pcurrent = frame_buffer.pb1;
    }
}


void bsp_lcd_set_display_buffer(uint8_t index)
{
    if(index == 1)
    {
        read_chg.gr_base = frame_buffer.pb1;
    }
    else
    {
        read_chg.gr_base = frame_buffer.pb2;
    }

    R_VDC_ChangeReadProcess(GRAPHICS_CHANNEL, VDC_LAYER_ID_2_RD, &read_chg);
    wait_vsync(1);
}
void bsp_lcd_set_working_buffer(uint8_t index)
{
    if(index == 1)
    {
        p = frame_buffer.pb1;
    }
    else
    {
        p = frame_buffer.pb2;
    }
}
