/**
 * @file m13_rza2m_ek_octaram.c
 * @brief M13-RZ/A2M-EK OctaRAM BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "m13_rza2m_ek_config.h"
#define TRACE_LEVEL BSP_CONFIG_TRACE_LEVEL_OCTARAM

// Dependencies
#include "r_octabus_lld_rza2m_api.h"
#include "m13_rza2m_ek_octaram.h"
#include "m13_rza2m_ek_vcom.h"

/**
 * @name       OctaBus_UserConfig
 * @brief      User defined device initialization routine for OctaBus
 * @param[in]  p_cfg : Pointer to the configuration table
 * @return     DRV_SUCCESS   Successful operation
 */
int_t OctaBus_UserConfig (const st_octabus_cfg_t *p_cfg)
{
    /* p_cfg not used */
    UNUSED_PARAM(p_cfg);

    st_octabus_configmode_t reg_config;
    uint32_t mode_value;

    //Debug message
    PRINTF_INFO(" Initializing OctaRAM...\r\n");

    // DCR Register configuration  : OctaRAM Device Mode Register Write command : 40h
    reg_config.dcr_value   = BSP_OCTARAM_DCSR_DVCMD1(0x40) |
    		                 BSP_OCTARAM_DCSR_DVCMD0(0x00);

    // DAR Register configuration : OctaRAM Device Mode Register Address : 00h 04h 00h 00h (See Device datasheet for full detail)
    reg_config.dar_value   = BSP_OCTARAM_DAR_DVAD3(0x00) |
							 BSP_OCTARAM_DAR_DVAD2(0x04) |
							 BSP_OCTARAM_DAR_DVAD1(0x00) |
							 BSP_OCTARAM_DAR_DVAD0(0x00);

    // DCSR Register configuration
    reg_config.dcsr_value  = BSP_OCTARAM_DCSR_ACDA(0)   |        // = 0b    : Register Access
    		                 BSP_OCTARAM_DCSR_DOPI(0)   |        // = 0b    : 1clock - 2byte
							 BSP_OCTARAM_DCSR_ADLEN(4)  |        // = 100b  : Address Length 4byte
							 BSP_OCTARAM_DCSR_DAOR(1)   |        // = 1b    : byte1, byte0, byte3, byte2
							 BSP_OCTARAM_DCSR_CMDLEN(2) |        // = 010b  : Command length 2byte
							 BSP_OCTARAM_DCSR_ACDV(1)   |        // = 1b    : Target Device1
							 BSP_OCTARAM_DCSR_DMLEN(0)  |        // = 000b  : Dummy cycle Length 0 clocks
						   	 BSP_OCTARAM_DCSR_DALEN(2);          // = 10b   : Data Length 2bytes

    // DCSTR Register configuration
	reg_config.dcstr_value = BSP_OCTARAM_DCSR_DVSELLO(0) |       // = 00   : 1.5 clock cycles
                             BSP_OCTARAM_DCSR_DVSELHI(5) |       // = 101  : 6.5 clock cycles
							 BSP_OCTARAM_DCSR_DVSELCMD(2);       // = 010  : 7 clock cycles

	// Set write data for OctaRAM Device Mode Register
	mode_value             = BSP_OCTARAM_DEVICE_DPD_ENABLE(1) |  // = 1    : (Default) Normal Operation
                             BSP_OCTARAM_DEVICE_DRIVE_STR(7)  |  // = 111  : (Default) 25ohms
                             BSP_OCTARAM_DEVICE_RFSH_RATE(0)  |  // = 00   : (Default) Always 4x Refresh
                             BSP_OCTARAM_DEVICE_LAT_CODE(2)   |  // = 0010 : 5 clocks / 133MHz
                             BSP_OCTARAM_DEVICE_LAT_TYPE(0)   |  // = 0    : (Default) Variable Latency
                             BSP_OCTARAM_DEVICE_BURST_TYPE(0) |  // = 0    : (Default) Wrapped
                             BSP_OCTARAM_DEVICE_BURST_LEN(1);    // = 01   : 64 bytes

    // Write to OctaRAM Device Mode Register
    R_OCTABUS_WriteConfigMode(&reg_config, mode_value);

    return DRV_SUCCESS;
}
