/**
 * @file m13_rza2m_ek_sdram.c
 * @brief M13-RZ/A2M-EK SDRAM BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/
#include "m13_rza2m_ek_config.h"
#define TRACE_LEVEL BSP_CONFIG_TRACE_LEVEL_SDRAM

//Dependencies
#include "iodefine.h"
#include "m13_rza2m_ek_sdram.h"
#include "m13_rza2m_ek_vcom.h"
#include "m13_rza2m_ek_status.h"

// Macro definitions

// Global variables and functions
int_t gpio_handle;


/**
 * @name       bsp_sdram_io_init
 * @brief      Initialize the SDRAM pins
 * @param[in]  None
 * @return     Error code (see m13_ra6m3_ek_status.h for enumeration)
 */
bsp_status_t bsp_sdram_io_init(void)
{
    int_t err;

    gpio_handle = direct_open("gpio", 0);
    if(gpio_handle < 0)
    {
		return BSP_STATUS_FAILURE;
    }

    err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_sdram_pin_table);

    if(err < 0)
    {
        return BSP_STATUS_PIN_CONFIG_FAILURE;
    }

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_sdram_init
 * @brief      Initialize the RZ/A2M gpio for the SDRAM device and configure the SDRAM settings
 * @param[in]  None
 * @return     None
 */
void bsp_sdram_init(void)
{
   //GPIO initialization
   bsp_sdram_io_init();

   //Debug message
   PRINTF_INFO(" Initializing SDRAM...\r\n");

   //CS3BCR settings
   BSC.CS3BCR.LONG          = BSC_CS3BCR_IWW_VAL(0)     |  //Idle Cycles between Write-Read Cycles and Write-Write Cycles (0 cycle)
                              BSC_CS3BCR_IWRWD_VAL(0)   |  //Idle Cycles for Another Space Read-Write (0 cycle)
                              BSC_CS3BCR_IWRWS_VAL(0)   |  //Idle Cycles for Read-Write in the Same Space (0 cycle)
                              BSC_CS3BCR_IWRRD_VAL(0)   |  //Idle Cycles for Read-Read in Another Space (0 cycle)
                              BSC_CS3BCR_IWRRS_VAL(0)   |  //Idle Cycles for Read-Read in the Same Space (0 cycle)
                              BSC_CS3BCR_TYPE_VAL(4)    |  //Type of memory connected to a space  (SDRAM)
                              BSC_CS3BCR_B11_VAL(1)     |  //Reserved
                              BSC_CS3BCR_BSZ_VAL(2);       //Data Bus Width Specification (16-bit)
													    
   //CS3WCR settings                                    
   BSC.CS3WCR.CS3WCR_1.LONG = BSC_CS3WCR_WTRP_VAL(1)    |  //Number of Auto-Precharge Completion Wait Cycles (1 cycle)
                              BSC_CS3WCR_WTRCD_VAL(1)   |  //Number of Wait Cycles between ACTV Command and READA/WRITA (1 cycle)
                              BSC_CS3WCR_A3CL_VAL(1)    |  //CAS Latency for Area 3 (2 cycles)
                              BSC_CS3WCR_TRWL_VAL(2)    |  //Number of Auto-Precharge Startup Wait Cycles (2 cycles)
                              BSC_CS3WCR_WTRC_VAL(2);      //Number of Idle Cycles from REF Command/Self-Refresh Release to ACTV/REF/MRS Command (5 cycles)
													    
   //SDCR settings                                      
   BSC.SDCR.LONG            = BSC_SDCR_A2ROW_VAL(0)     |  //Number of Bits of Row Address for Area 2 (N/A)
                              BSC_SDCR_A2COL_VAL(0)     |  //Number of Bits of Column Address for Area 2 (N/A)
                              BSC_SDCR_DEEP_VAL(0)      |  //Deep Power-Down Mode (self-refresh mode)
                              BSC_SDCR_RFSH_VAL(1)      |  //Refresh Control (refresh)
                              BSC_SDCR_RMODE_VAL(0)     |  //Refresh Control (auto-refresh is performed)
                              BSC_SDCR_PDOWN_VAL(0)     |  //Power-Down Mode (no power-down mode)
                              BSC_SDCR_BACTV_VAL(0)     |  //Bank Active Mode (auto-precharge mode)
                              BSC_SDCR_A3ROW_VAL(2)     |  //Number of Bits of Row Address for Area 3 (12 bits) MODIF PAT 13 bits
                              BSC_SDCR_A3COL_VAL(1);       //Number of Bits of Column Address for Area 3 (9 bits)

   //RTCOR settings
   BSC.RTCOR                = BSC_RTCOR_KEY_VAL(0xA55A) |  //Key
                              BSC_RTCSR_RTCSR_VAL(32);     //32 cycles per refresh

   //RTCSR settings
   BSC.RTCSR.LONG           = BSC_RTCSR_KEY_VAL(0xA55A) |  //Key
                              BSC_RTCSR_CMF_VAL(0)      |  //Compare Match Flag (clearing condition)
                              BSC_RTCSR_CMIE_VAL(0)     |  //Compare Match Interrupt Enable (disable CMF interrupts)
                              BSC_RTCSR_CKS_VAL(2)      |  //Clock Select (CKIO-PHY/16)
                              BSC_RTCSR_RRC_VAL(0);        //Refresh Count (1 time)

   //Dump SDRAM configuration
   PRINTF_DEBUG("  CS3BCR = 0x%08X ...\r\n", BSC.CS3BCR);
   PRINTF_DEBUG("  CS3WCR = 0x%08X ...\r\n", BSC.CS3WCR);
   PRINTF_DEBUG("  SDCR   = 0x%08X ...\r\n", BSC.SDCR);
   PRINTF_DEBUG("  RTCOR  = 0x%08X ...\r\n", BSC.RTCOR);
   PRINTF_DEBUG("  RTCSR  = 0x%08X ...\r\n", BSC.RTCSR);

   //Write to SDRAM mode register (burst read/burst write, 16-bit data bus, CAS latency 2)
   SDRAM_MODE_CS3 = 0x0000;
}

