/**
 * @file m13_rza2m_ek_touch.c
 * @brief M13-RZ/A2M-EK touchscreen BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http:// www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "m13_rza2m_ek_config.h"
#define TRACE_LEVEL BSP_TRACE_LEVEL_TOUCH

// Dependencies
#include "m13_rza2m_ek_touch.h"
#include "m13_rza2m_ek_vcom.h"

// Macro definitions
#define TOUCH_FT5206_W 0x70
#define TOUCH_FT5206_R 0x71
#define RIIC_CH3_RZA2M (DEVICE_INDENTIFIER "riic3")

// Global variables
static int_t gpio_handle;
static int_t touch_handle;
int32_t touch_read_request = 0;

static void int_irq_touch_panel(uint32_t int_sense);

/**
 * @name       bsp_touch_init
 * @brief      Configure Touch IRQ and Touchscreen in Normal Mode
 * @param[in]  None
 * @return     Error code
 */
bsp_status_t bsp_touch_init(void)
{
	int_t err;
    uint8_t value;
    bsp_status_t error;

    // Open I2C channel 3 and initialize Touch handler
    touch_handle = bsp_i2c_open(BSP_I2C_CHANNEL_3);

	if (touch_handle == BSP_I2C_INVALID_HANDLE)
	{
		return BSP_STATUS_INVALID_CHANNEL;
	}

    gpio_handle = direct_open("gpio", 0);

    if(gpio_handle < 0)
    {
		return BSP_STATUS_FAILURE;
    }

    err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_irq1_pin_table);

    if(err < 0)
    {
		return BSP_STATUS_PIN_CONFIG_FAILURE;
    }

    // Configure touch interrupt as IRQ1

    INTC.ICR1.BIT.IRQ10S = 0;                                      // IRQ1 set to falling edge
    INTC.ICR1.BIT.IRQ11S = 1;                                      // IRQ1 set to falling edge

    // IRQ1 Register settings
    R_INTC_Disable(INTC_ID_IRQ_IRQ1);                              // Disable IRQ1
    R_INTC_RegistIntFunc(INTC_ID_IRQ_IRQ1, &int_irq_touch_panel);  // Register interrupt handler
    R_INTC_Enable(INTC_ID_IRQ_IRQ1);                               // Enable IRQ1
    R_INTC_SetPriority(INTC_ID_IRQ_IRQ1, 8);                       // Set Priority 8

    // Force Touch in Normal Mode
    error = bsp_touch_write_reg(FT5206_DEVICE_MODE, FT5206_DEVICE_MODE_MODE_NORMAL);

    // If Touch Normal Mode could not be written, return error code
    if(error == BSP_STATUS_FAILURE)
    {
        PRINTF_INFO("Error could not write Touch Mode \r\n");
        return error;
    }

    // Check device mode
    bsp_touch_read_reg(FT5206_DEVICE_MODE, &value);

    // Return error code if wrong device mode detected
    if(value != FT5206_DEVICE_MODE_MODE_NORMAL)
    {
        PRINTF_DEBUG("Wrong device mode = 0x%02X instead of 0x00 \r\n", value);
        return BSP_STATUS_INVALID_VERSION;
    }

    PRINTF_INFO(" Touch Init Success! \r\n");

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_get_touch_status
 * @brief      Check Touch Status Register
 * @param[in]  touch_status
 * @return     Error code
 */
bsp_status_t bsp_touch_get_status(uint8_t *touch_status)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, FT5206_TD_STATUS, 1, touch_status, 1);

    return error;
}

/**
 * @name       bsp_get_gest_id
 * @brief      Check gesture after a valid touch is detected
 * @param[in]  touch_status
 * @return     Error code
 */
bsp_status_t bsp_touch_get_gest_id(uint8_t *gest_id)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, FT5206_GEST_ID, 1, gest_id, sizeof(gest_id) );

    return error;
}

/**
 * @name       bsp_touch_get_data
 * @brief      Retrieve all the touch data from registers
 * @param[in]  Data table
 * @return     Error code
 */
bsp_status_t bsp_touch_get_data(uint8_t *touch_data, size_t size)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, BSP_I2C_SLAVE_ADDR_TOUCH, FT5206_DEVICE_MODE, 1, touch_data, size);

    return error;
}

/**
 * @name       bsp_get_finger_position
 * @brief      Retrieve and format the x,y position of requested finger
 * @param[in]  Touch data pointer
 * @param[in]  Finger ID (position)
 * @param[in]  x data pointer
 * @param[in]  y data pointer
 * @return     Error code
 */
bsp_status_t bsp_touch_get_position(uint8_t *touch_data, uint8_t finger_id, uint16_t *x, uint16_t *y)
{
    if(finger_id >= 1 && finger_id <= 5)
    {
        // Form X1 touch position data to 16bits
        *x = ( (touch_data[FT5206_TOUCH_XH(finger_id)] & FT5206_TOUCH_XH_MSB_X_POS) << 8)
            + touch_data[FT5206_TOUCH_XL(finger_id)];

        // Form Y1 touch position data to 16bits
        *y = ( (touch_data[FT5206_TOUCH_YH(finger_id)] & FT5206_TOUCH_YH_MSB_Y_POS) << 8)
            + touch_data[FT5206_TOUCH_YL(finger_id)];
    }
    else
    {
        *x = 0;
        *y = 0;
    }

    if(*x > 799)
        *x = 799;

    if(*y > 479)
        *y = 479;

    return BSP_STATUS_OK;
}

/**
 * @name       bsp_touch_get_finger_event
 * @brief      Check if there is an finger event detected
 * @param[in]  Touch data pointer
 * @param[in]  Finger ID (position)
 * @param[in]  Finger event data pointer 
 * @return     Error code
 */
bsp_status_t bsp_touch_get_finger_event(uint8_t *touch_data, uint8_t finger_id, uint8_t *event)
{
    // To do
	return BSP_STATUS_OK;
}

/**
 * @name       bsp_touch_write_reg
 * @brief      Write touchscreen register
 * @param[in]  Register address
 * @param[in]  value to be written
 * @return     Error code
 */
bsp_status_t bsp_touch_write_reg(uint8_t address, uint8_t data)
{
    bsp_status_t error;

    error = bsp_i2c_write (touch_handle, TOUCH_FT5206_W, address, 1, &data, 1 );

    // Return status code
    return error;
}

/**
 * @name       bsp_touch_read_reg
 * @brief      Read touchscreen register
 * @param[in]  Register address
 * @param[in]  Value to receive data read
 * @return     Error code
 */
bsp_status_t bsp_touch_read_reg(uint8_t address, uint8_t *data)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, TOUCH_FT5206_R, address, 1, data, 1 );

    // Return status code
    return error;
}

/**
 * @name       bsp_touch_read_reg
 * @brief      Read touchscreen register
 * @param[in]  Register address
 * @param[in]  Value to receive data read
 * @param[in]  data size to be read
 * @return     Error code
 */
bsp_status_t bsp_touch_read_regs(uint8_t address, uint8_t *data, size_t data_size)
{
    bsp_status_t error;

    error = bsp_i2c_read (touch_handle, TOUCH_FT5206_R, address, 1, data, data_size);

    // Return status code
    return error;
}

/**
 * @name       int_irq_touch_panel
 * @brief      Interrupt service routine : Clear IRQ flag after touch event detected
 * @param[in]  IRQ Handle (Not used)
 * @return     Error code
 */
static void int_irq_touch_panel(uint32_t int_sense)
{
    UNUSED_PARAM(int_sense);

    touch_read_request++;

    INTC.IRQRR.BIT.IRQ1F = 0; // Clear interrupt flag

}
