/**
 * @file main.c
 * @brief main file
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Dependencies
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>


#include "r_cpg_drv_api.h"
#include "r_ostm_drv_api.h"
#include "r_startup_config.h"
#include "r_os_abstraction_api.h"
#include "r_task_priority.h"
#include "r_octabus_lld_rza2m_api.h"

#include "freertos.h"
#include "task.h"
#include "semphr.h"

#include "iodefine.h"
#include "compiler_settings.h"
#include "main.h"

#include "m13_rza2m_ek_lcd.h"
#include "m13_rza2m_ek_button.h"
#include "m13_rza2m_ek_led.h"
#include "m13_rza2m_ek_acc.h"
#include "m13_rza2m_ek_vcom.h"
#include "m13_rza2m_ek_status.h"
#include "m13_rza2m_ek_i2c.h"
#include "m13_rza2m_ek_touch.h"
#include "m13_rza2m_ek_octaram.h"
#include "m13_rza2m_ek_sdram.h"
#include "m13_rza2m_ek_eeprom.h"

// Macro definitions
#define MAIN_PRV_LED_ON        (1)
#define MAIN_PRV_LED_OFF       (0)
#define TOLERANCE              200
#define THRESHOLD              1700

// Global variables
int16_t ax;
int16_t ay;
int16_t az;
int8_t  ay_toggle = 0;
uint8_t touch_status;
uint8_t touch_data[31];

// Imported global variables (from other files)
extern uint16_t background_color;
extern uint16_t screen_rotation;

// private variables
static uint32_t gs_main_led_flg;
static SemaphoreHandle_t i2c3_event = NULL;

// Private global functions
void switch_task(void *parameters);
void switch_task_start(void);
void adc_task(void *parameters);
void adc_task_start(void);
void lcd_task(void *parameters);
void lcd_task_start(void);
void mems_task(void *parameters);
void mems_task_start(void);
void touch_task(void *parameters);
void touch_task_start(void);

void lcd_rotation_check(uint16_t *x_rot, uint16_t *y_rot);
void lcd_display_test_screen(void);
void lcd_draw_square_under_finger(uint16_t x_touch_rot, uint16_t y_touch_rot);


// Imported global functions (from other files)
extern void cmd_console(FILE *pIn, FILE *pOut, char_t *pszPrompt);
extern int32_t touch_read_request;

/**
 * @name       os_console_task_t
 * @brief      console task
 * @param[in]  None
 * @return     None
 */
int_t os_console_task_t(void)
{
    /* never exits */

    while(1)
    {
        /* ==== Receive command, activate sample software ==== */
        cmd_console(stdin, stdout, "SAMPLE>");

        R_OS_TaskSleep(500);
    }
}


//extern void show_welcome_msg (FILE *p_out, bool_t clear_screen);
/**
 * @name       os_main_task_t
 * @brief      FreeRTOS main task called by R_OS_KernelInit()
 * @param[in]  None
 * @return     None
 */
int_t os_main_task_t(void)
{
    // Initialize led
    bsp_led_init();

    // Configure and Initialize switch pins
    bsp_button_init();

    // Initialize VCOM
    bsp_vcom_init();

    // Initialize the LCD
    bsp_lcd_init();
    bsp_lcd_clear(LCD_DEFAULT_BACKGROUND_COLOR);
    bsp_lcd_set_text_background_color(LCD_DEFAULT_BACKGROUND_COLOR);
    bsp_lcd_set_text_color(LCD_DEFAULT_TEXT_COLOR);
    bsp_lcd_set_font(&font_22x29);
    screen_rotation = ROTATION_ZERO;
    bsp_lcd_display_on();

    // Initialize I2C channel 3
    bsp_i2c_init(BSP_I2C_CHANNEL_3);

    // eeprom init
    bsp_eeprom_init();
    bsp_eeprom_test();

    // Configure and start touch
    bsp_touch_init();

    // Configure accelerometer
    bsp_acc_init();

    // Create Mutex for I2C3 events
    i2c3_event = xSemaphoreCreateMutex();

    // Init OctaRAM
    R_OCTABUS_Setup();

    // Init SDRAM
    bsp_sdram_init();

    // Launch all the tasks
    lcd_task_start();

    switch_task_start();

    touch_task_start();

    mems_task_start();

    // Start-up message
    PRINTF_INFO("\r\n");
    PRINTF_INFO(" ***********************************\r\n");
    PRINTF_INFO(" ****** M13-RZA2M-EK BSP DEMO ******\r\n");
    PRINTF_INFO(" ***********************************\r\n");
    PRINTF_INFO(" Copyright: 2010-2021 M13design SARL\r\n");
    PRINTF_INFO(" Compiled: %s %s\r\n", __DATE__, __TIME__);
    PRINTF_INFO(" Target: RZ/A2M\r\n");
    PRINTF_INFO("\r\n");

    // Main task is used for blinking LED1
    while(1)
    {
        gs_main_led_flg ^= 1;

        if(MAIN_PRV_LED_ON == gs_main_led_flg)
        {
            bsp_led_off(LED1);
            vTaskDelay(900);
        }
        else
        {
            bsp_led_on(LED1);
            vTaskDelay(100);
        }
    }
}

/**
 * @name       main
 * @brief      C Entry point
 *             opens and configures cpg driver
 *             starts the freertos kernel
 * @param[in]  None
 * @return     None
 */
int_t main(void)
{
    int_t cpg_handle;

    /* Initialize the devlink layer */
     R_DEVLINK_Init();

    /* Initialize CPG */
    cpg_handle = direct_open("cpg", 0);
    if(cpg_handle < 0)
    {
        /* stop execute */
        while(1)
        {
            ;
        }
    }

    /* Can close handle if no need to change clock after here */
    direct_close(cpg_handle);

    /* Start FreeRTOS */
    /* R_OS_InitKernel should never return */
    R_OS_KernelInit();
}

/**
 * @name       switch_task
 * @brief      User Switch Task application
 * @param[in]  None
 * @return     None
 */
void switch_task(void *parameters)
{
    while(1)
    {
    	// To do
        vTaskDelay(200);
    }
}

/**
 * @name       switch_task_start
 * @brief      Task creation for switch_task
 * @param[in]  None
 * @return     None
 */
void switch_task_start(void)
{
    portBASE_TYPE status;
    xTaskHandle task = NULL;

    status = xTaskCreate(switch_task, "Switch Task", 1000, NULL, 1, &task);

    //Failed to create the task?
    if(task == BSP_INVALID_HANDLE)
    {
        //Debug message
        PRINTF_INFO("Failed to create task!\r\n");
    }
}

/**
 * @name       lcd_task
 * @brief      User LCD Task application
 * @param[in]  None
 * @return     None
 */
void lcd_task(void *parameters)
{
    uint16_t x_pos = 0;
    uint16_t y_pos = 0;
    uint16_t x_cen = 0;
    uint16_t y_cen = 0;
    uint16_t x_touch_rot = 0;
    uint16_t y_touch_rot = 0;
    uint16_t x_length = 0;
    uint16_t y_length = 0;


    // clear touch data
    memset(touch_data, 0x00, sizeof(touch_data) );

    // Enable double buffer
    bsp_lcd_enable_double_buffer();

    while(1)
    {
        // Check and refresh if screen is rotated 0 or 180°
        lcd_rotation_check(&x_touch_rot, &y_touch_rot);

        // Wait for an i2c event
        xSemaphoreTake(i2c3_event, portMAX_DELAY);

        // Process touch screen data if touch irq is asserted
        if(touch_read_request > 0)
        {
            // Check touch status
            bsp_touch_get_status(&touch_status);

            // If status is valid, process touch data
            if(touch_status >= 1 && touch_status <= 5)
            {
                bsp_touch_get_data(touch_data, sizeof(touch_data));
            }

            // Reset touch irq flag
            touch_read_request = 0;
        }

        // Release semaphore
        xSemaphoreGive(i2c3_event);

        // Draw test screen
        lcd_display_test_screen();

        // Draw a yellow square under each finger detected on screen
        if(touch_status >= 1 && touch_status <= 5)
        {
            lcd_draw_square_under_finger(x_touch_rot, y_touch_rot);
        }

        bsp_lcd_swap_buffer();

        vTaskDelay(1);
    }
}

/**
 * @name       lcd_task_start
 * @brief      Task creation for lcd_task
 * @param[in]  None
 * @return     None
 */
void lcd_task_start(void)
{
    portBASE_TYPE status;
    xTaskHandle task = NULL;

    status = xTaskCreate(lcd_task, "LCD Task", 2000, NULL, 1, &task);

    //Failed to create the task?
    if(task == BSP_INVALID_HANDLE)
    {
        //Debug message
        PRINTF_INFO("Failed to create task!\r\n");
    }
}

/**
 * @name       mems_task
 * @brief      User mems Task application
 * @param[in]  None
 * @return     None
 */
void mems_task(void *parameters)
{
    while(1)
    {
        xSemaphoreTake(i2c3_event, portMAX_DELAY);

        bsp_acc_get_acc(&ax, &ay, &az);

        xSemaphoreGive(i2c3_event);

        //PRINTF_DEBUG(" ax = %d \r\n", ax);
        PRINTF_DEBUG(" ay = %d \r\n", ay);
        //PRINTF_DEBUG(" az = %d \r\n", az);

        vTaskDelay(50);
    }
}

/**
 * @name       mems_task_start
 * @brief      Task creation for mems_task
 * @param[in]  None
 * @return     None
 */
void mems_task_start(void)
{
    portBASE_TYPE status;
    xTaskHandle task = NULL;

    status = xTaskCreate(mems_task, "MEM Task", 2000, NULL, 1, &task);

    //Failed to create the task?
    if(task == BSP_INVALID_HANDLE)
    {
        //Debug message
        PRINTF_INFO("Failed to create task!\r\n");
    }
}


/**
 * @name       mems_task
 * @brief      User mems Task application
 * @param[in]  None
 * @return     None
 */
void touch_task(void *parameters)
{
    uint8_t touch_status;
    uint8_t touch_data[31];

    uint16_t x_pos[5];
    uint16_t y_pos[5];

    // clear touch data
    memset(touch_data, 0x00, sizeof(touch_data) );

    while(1)
    {
        //Wait for an event
        xSemaphoreTake(i2c3_event, portMAX_DELAY);

        if(touch_read_request > 0)
        {
            bsp_touch_get_status(&touch_status);

            if(touch_status >= 1 && touch_status <= 5)
            {
                bsp_touch_get_data(touch_data, sizeof(touch_data));
            }
            // Reset irq flag
            touch_read_request = 0;
        }

        // Release semaphore
        xSemaphoreGive(i2c3_event);

        // Format and display Finger coords on VCOM
        if(touch_status >= 1 && touch_status <= 5)
        {
            // Retrieve and form finger position data
            for(uint8_t i = 0; i < touch_status; i++)
            {
                bsp_touch_get_position(touch_data, ID_FINGER(i), &x_pos[i], &y_pos[i]);
            }

            PRINTF_INFO("\r\n Number of finger detected  = %0d \r\n", ID_FINGER(touch_status - 1));

            PRINTF_INFO("       T1|   T2|   T3|   T4|   T5| \r\n");
            PRINTF_INFO(" X =");

            for(uint8_t i = 0; i < touch_status; i++)
            {
                if(screen_rotation == ROTATION_ZERO)
                {
                    PRINTF_INFO("  %3d|", x_pos[i]);
                }
                else if(screen_rotation == ROTATION_180)
                {
                    PRINTF_INFO("  %3d|", abs(x_pos[i] - LCD_WIDTH) );
                }
            }

            PRINTF_INFO("\r\n");
            PRINTF_INFO(" Y =");

            for(uint8_t i = 0; i < touch_status; i++)
            {
                if(screen_rotation == ROTATION_ZERO)
                {
                    PRINTF_INFO("  %3d|", y_pos[i]);
                }
                else if(screen_rotation == ROTATION_180)
                {
                    PRINTF_INFO("  %3d|", abs(y_pos[i] - LCD_HEIGHT) );
                }
            }

            PRINTF_INFO("\r\n");


            touch_status = 0;
        }

        vTaskDelay(500);
    }
}

/**
 * @name       mems_task_start
 * @brief      Task creation for mems_task
 * @param[in]  None
 * @return     None
 */
void touch_task_start(void)
{
    portBASE_TYPE status;
    xTaskHandle task = NULL;

    status = xTaskCreate(touch_task, "Touch Task", 2000, NULL, 1, &task);

    //Failed to create the task?
    if(task == BSP_INVALID_HANDLE)
    {
        //Debug message
        PRINTF_INFO("Failed to create task!\r\n");
    }
}

void vApplicationStackOverflowHook(xTaskHandle pxTask, char *pcTaskName)
{
    (void) pcTaskName;
    (void) pxTask;

    taskDISABLE_INTERRUPTS();
    while(1)
    {
        ;
    }
}


void lcd_rotation_check(uint16_t *x_rot, uint16_t *y_rot)
{
#if 0
    if( (ay > (THRESHOLD + TOLERANCE) ) && (ay_toggle == 0) )
    {
        screen_rotation = ROTATION_180;

        bsp_lcd_clear(background_color);

        ay_toggle = 1;

        *x_rot = LCD_WIDTH;
        *y_rot = LCD_HEIGHT;

        PRINTF_INFO(" Screen ROTATION_180 \r\n");
    }
    else if( (ay <= (THRESHOLD) ) && (ay_toggle == 1) )
    {
        screen_rotation = ROTATION_ZERO;

        bsp_lcd_clear(background_color);

        ay_toggle = 0;

        *x_rot = 0;
        *y_rot = 0;

        PRINTF_INFO(" Screen ROTATION_ZERO \r\n");
    }
    else
    {

    }
#else
    if( (ay > (THRESHOLD + TOLERANCE) ) && (ay_toggle == 0) )
    {
    	vTaskDelay(100);

        if( ay > (THRESHOLD + TOLERANCE) )
        {
        	vTaskDelay(100);

            if( ay > (THRESHOLD + TOLERANCE) )
            {
				screen_rotation = ROTATION_180;

				bsp_lcd_clear(background_color);

				ay_toggle = 1;

				*x_rot = LCD_WIDTH;
				*y_rot = LCD_HEIGHT;

				PRINTF_INFO(" Screen ROTATION_180 \r\n");
            }
        }

    }
    else if( (ay <= (THRESHOLD) ) && (ay_toggle == 1) )
    {
    	vTaskDelay(200);

        screen_rotation = ROTATION_ZERO;

        bsp_lcd_clear(background_color);

        ay_toggle = 0;

        *x_rot = 0;
        *y_rot = 0;

        PRINTF_INFO(" Screen ROTATION_ZERO \r\n");
    }
    else
    {

    }
#endif
}

void lcd_display_test_screen(void)
{
    // clear screen with background color
    bsp_lcd_clear(LCD_DEFAULT_BACKGROUND_COLOR);

    // Set font and color for default texts
    bsp_lcd_set_text_color(LCD_COLOR_BLACK);
    bsp_lcd_set_font(&font_22x29);

    // Display default texts
    bsp_lcd_display_string(301, 60, "M13DESIGN");
    bsp_lcd_display_string(268, 150, "M13-RZA2M-EK");
    bsp_lcd_display_string(312, 300, "BSP DEMO");
    bsp_lcd_draw_rect(250, 292, 300, 40, LCD_COLOR_BLACK);

    // Set font and color for coords. text
    bsp_lcd_set_text_color(LCD_COLOR_BLUE);
    bsp_lcd_set_font(&font_14x19);

    // Display corner's coordinates
    bsp_lcd_display_string(1, 5, "(0,0)");
    bsp_lcd_display_string(702, 5, "(800,0)");
    bsp_lcd_display_string(1, 461, "(0,480)");
    bsp_lcd_display_string(674, 461, "(800,480)");
}

void lcd_draw_square_under_finger(uint16_t x_touch_rot, uint16_t y_touch_rot)
{
    uint16_t x_pos = 0;
    uint16_t y_pos = 0;
    uint16_t x_cen = 0;
    uint16_t y_cen = 0;
    uint16_t x_length = 0;
    uint16_t y_length = 0;

    for(uint8_t i = 0; i < touch_status; i++)
    {
        bsp_touch_get_position(touch_data, ID_FINGER(i), &x_pos, &y_pos);

        x_cen = abs(x_pos - x_touch_rot);
        y_cen = abs(y_pos - y_touch_rot);

        if(x_cen < 50)
        {
            x_length = 50 + x_cen;
            x_cen = 50;
        }
        else
        {
            x_length = 100;
        }

        if(y_cen < 50)
        {
            y_length = 50 + y_cen;
            y_cen = 50;

        }
        else
        {
            y_length = 100;
        }

        // Draw a yellow square
        bsp_lcd_fill_rect(x_cen - 50, y_cen - 50, x_length, y_length, LCD_COLOR_YELLOW);
    }
}

// End of File
